@@ -23,12 +23,21 @@ signing_methods = {
 }
 
 def base64url_decode(input):
-    input += '=' * (4 - (len(input) % 4))
+    rem = len(input) % 4
+    if rem > 0:
+        input += '=' * (4 - rem)
     return base64.urlsafe_b64decode(input)
 
 def base64url_encode(input):
     return base64.urlsafe_b64encode(input).replace('=', '')
 
+def header(jwt):
+    header_segment = jwt.split('.', 1)[0]
+    try:
+        return json.loads(base64url_decode(header_segment))
+    except (ValueError, TypeError):
+        raise DecodeError("Invalid header encoding")
+
 def encode(payload, key, algorithm='HS256'):
     segments = []
     header = {"typ": "JWT", "alg": algorithm}
@@ -36,8 +45,9 @@ def encode(payload, key, algorithm='HS256'):
     segments.append(base64url_encode(json.dumps(payload)))
     signing_input = '.'.join(segments)
     try:
-        ascii_key = unicode(key).encode('utf8')
-        signature = signing_methods[algorithm](signing_input, ascii_key)
+        if isinstance(key, unicode):
+            key = key.encode('utf-8')
+        signature = signing_methods[algorithm](signing_input, key)
     except KeyError:
         raise NotImplementedError("Algorithm not supported")
     segments.append(base64url_encode(signature))
@@ -45,7 +55,7 @@ def encode(payload, key, algorithm='HS256'):
 
 def decode(jwt, key='', verify=True):
     try:
-        signing_input, crypto_segment = jwt.rsplit('.', 1)
+        signing_input, crypto_segment = str(jwt).rsplit('.', 1)
         header_segment, payload_segment = signing_input.split('.', 1)
     except ValueError:
         raise DecodeError("Not enough segments")
@@ -57,8 +67,9 @@ def decode(jwt, key='', verify=True):
         raise DecodeError("Invalid segment encoding")
     if verify:
         try:
-            ascii_key = unicode(key).encode('utf8')
-            if not signature == signing_methods[header['alg']](signing_input, ascii_key):
+            if isinstance(key, unicode):
+                key = key.encode('utf-8')
+            if not signature == signing_methods[header['alg']](signing_input, key):
                 raise DecodeError("Signature verification failed")
         except KeyError:
             raise DecodeError("Algorithm not supported")
